<?xml version='1.0' encoding='UTF-8'?>

<!--
   NIPS C - New Implementation of Promela Semantics Compiler
    XML tree transformation package
    based on work of: Peter Eisentraut <peter_e@gmx.net>
                      Gustavo Quiros <gustavo.quiros@rwth-aachen.de>
                      Michael Rohrbach <m.rohrbach00@gmx.de>
                      Lehrstuhl fuer Informatik II, RWTH Aachen
    corrected and improved by: Stefan Schuermans <stefan@schuermans.info>
                               Lehrstuhl fuer Informatik II, RWTH Aachen
    Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html

    Promela Parse tree to AST conversion
    
    These rules match the important elements of the XML parse tree, and produce
    their equivalents in the AST, as defined in the file 'promela-ast.dtd'. All
    other elements of the parse tree are ignored.
-->


<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="xml" doctype-system="classpath://xml/promela-ast.dtd"/> 

<xsl:strip-space elements="*"/> 

<xsl:template match="@line">
    <xsl:copy/>
</xsl:template>

<xsl:template match="@column">
    <xsl:copy/>
</xsl:template>

<xsl:template match="promela-parse">
    <promela-ast source="{@source}">
        <xsl:apply-templates select="*|@line|@column"/>
    </promela-ast>
</xsl:template>

<xsl:template match="utype">
    <utype>
        <xsl:apply-templates select="*|@line|@column"/>
    </utype>
</xsl:template>

<xsl:template match="mtype">
    <mtype>
	<xsl:apply-templates select="@line|@column"/>
        <xsl:for-each select=".//t_name">
            <mtype-name value="{@text}">
		<xsl:apply-templates select="@line|@column"/>
	    </mtype-name>
        </xsl:for-each>
    </mtype>
</xsl:template>

<xsl:template match="proctype">
    <proctype>
        <xsl:attribute name="active">0</xsl:attribute>
        <xsl:apply-templates select="*|@line|@column"/>
    </proctype>
</xsl:template>

<xsl:template match="proctype_args">
    <args>
        <xsl:apply-templates select="*|@line|@column"/>
    </args>
</xsl:template>

<xsl:template match="enabler">
    <enabler>
        <xsl:apply-templates select="*|@line|@column"/>
    </enabler>
</xsl:template>

<xsl:template match="init">
    <init>
        <xsl:apply-templates select="*|@line|@column"/>
    </init>
</xsl:template>

<xsl:template match="never">
    <never>
        <xsl:apply-templates select="*|@line|@column"/>
    </never>
</xsl:template>

<xsl:template match="trace_trace">
    <trace>
        <xsl:apply-templates select="*|@line|@column"/>
    </trace>
</xsl:template>

<xsl:template match="notrace_trace">
    <notrace>
        <xsl:apply-templates select="*|@line|@column"/>
    </notrace>
</xsl:template>

<xsl:template match="c_code_module">
    <c-code>
        <xsl:apply-templates select="*|@line|@column"/>
    </c-code>
</xsl:template>

<xsl:template match="c_decl_module">
    <c-decl>
        <xsl:apply-templates select="*|@line|@column"/>
    </c-decl>
</xsl:template>

<xsl:template match="c_state_module">
    <c-state string1="{t_string[1]/@text}" string2="{t_string[2]/@text}">
	<xsl:for-each select="t_string[3]">
	   <xsl:attribute name="string3"><xsl:value-of select="@text"/></xsl:attribute>
	</xsl:for-each>
        <xsl:apply-templates select="@line|@column"/>
    </c-state>
</xsl:template>

<xsl:template match="c_track_module">
    <c-track string1="{t_string[1]/@text}" string2="{t_string[2]/@text}">
        <xsl:apply-templates select="@line|@column"/>
    </c-track>
</xsl:template>

<xsl:template match="typename_one_decl">
    <decl>
        <xsl:apply-templates select="*|@line|@column"/>
    </decl>
</xsl:template>

<xsl:template match="unsigned_one_decl">
    <decl>
        <xsl:apply-templates select="*|@line|@column"/>
    </decl>
</xsl:template>

<xsl:template match="bit_typename">
    <xsl:attribute name="typename">bit</xsl:attribute>
</xsl:template>

<xsl:template match="bit_typename" mode="ch-init">
    <typename name="bit"><xsl:apply-templates select="@line|@column"/></typename>
</xsl:template>

<xsl:template match="bool_typename">
    <xsl:attribute name="typename">bool</xsl:attribute>
</xsl:template>

<xsl:template match="bool_typename" mode="ch-init">
    <typename name="bool"><xsl:apply-templates select="@line|@column"/></typename>
</xsl:template>

<xsl:template match="byte_typename">
    <xsl:attribute name="typename">byte</xsl:attribute>
</xsl:template>

<xsl:template match="byte_typename" mode="ch-init">
    <typename name="byte"><xsl:apply-templates select="@line|@column"/></typename>
</xsl:template>

<xsl:template match="pid_typename">
    <xsl:attribute name="typename">pid</xsl:attribute>
</xsl:template>

<xsl:template match="pid_typename" mode="ch-init">
    <typename name="pid"><xsl:apply-templates select="@line|@column"/></typename>
</xsl:template>

<xsl:template match="short_typename">
    <xsl:attribute name="typename">short</xsl:attribute>
</xsl:template>

<xsl:template match="short_typename" mode="ch-init">
    <typename name="short"><xsl:apply-templates select="@line|@column"/></typename>
</xsl:template>

<xsl:template match="int_typename">
    <xsl:attribute name="typename">int</xsl:attribute>
</xsl:template>

<xsl:template match="int_typename" mode="ch-init">
    <typename name="int"><xsl:apply-templates select="@line|@column"/></typename>
</xsl:template>

<xsl:template match="mtype_typename">
    <xsl:attribute name="typename">mtype</xsl:attribute>
</xsl:template>

<xsl:template match="mtype_typename" mode="ch-init">
    <typename name="mtype"><xsl:apply-templates select="@line|@column"/></typename>
</xsl:template>

<xsl:template match="chan_typename">
    <xsl:attribute name="typename">chan</xsl:attribute>
</xsl:template>

<xsl:template match="chan_typename" mode="ch-init">
    <typename name="chan"><xsl:apply-templates select="@line|@column"/></typename>
</xsl:template>

<xsl:template match="uname_typename">
    <xsl:attribute name="typename">
        <xsl:value-of select=".//t_name/@text"/>
    </xsl:attribute>
</xsl:template>

<xsl:template match="uname_typename" mode="ch-init">
    <typename name="{.//t_name/@text}"><xsl:apply-templates select="@line|@column"/></typename>
</xsl:template>

<xsl:template match="t_unsigned">
    <xsl:attribute name="typename">unsigned</xsl:attribute>
</xsl:template>

<xsl:template match="active">
    <xsl:attribute name="active">1</xsl:attribute>
        <xsl:for-each select="const_with_brackets">
            <xsl:attribute name="active">
                <xsl:value-of select="t_const/@text"/>
            </xsl:attribute>
        </xsl:for-each>
</xsl:template>

<xsl:template match="priority">
    <xsl:attribute name="priority">
        <xsl:value-of select=".//t_const/@text"/>
    </xsl:attribute>
</xsl:template>

<xsl:template match="hidden_visible">
    <xsl:attribute name="visible">hidden</xsl:attribute>
</xsl:template>

<xsl:template match="show_visible">
    <xsl:attribute name="visible">show</xsl:attribute>
</xsl:template>

<xsl:template match="xr_step">
    <xr>
        <xsl:apply-templates select="*|@line|@column"/>
    </xr>
</xsl:template>

<xsl:template match="xs_step">
    <xs>
        <xsl:apply-templates select="*|@line|@column"/>
    </xs>
</xsl:template>

<xsl:template match="stmnts_step">
    <xsl:choose>
        <xsl:when test="unless_stmnt">
            <unless>
                <xsl:apply-templates select="*|@line|@column"/>
            </unless>
        </xsl:when>
        <xsl:otherwise>
            <xsl:apply-templates select="*"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>


<xsl:template match="ivar">
    <ivar>
        <xsl:for-each select="const_with_brackets">
            <xsl:attribute name="array-size">
                <xsl:value-of select="t_const/@text"/>
            </xsl:attribute>
        </xsl:for-each>
        <xsl:apply-templates select="*|@line|@column"/>
    </ivar>
</xsl:template>

<xsl:template match="uivar">
    <uivar>
        <xsl:attribute name="bits">
            <xsl:value-of select=".//t_const/@text"/>
        </xsl:attribute>
        <xsl:for-each select="const_with_brackets">
            <xsl:attribute name="array-size">
                <xsl:value-of select="t_const/@text"/>
            </xsl:attribute>
        </xsl:for-each>
        <xsl:apply-templates select="*|@line|@column"/>
    </uivar>
</xsl:template>

<xsl:template match="ch_init">
    <ch-init array-size="{t_const/@text}">
	<xsl:apply-templates select="@line|@column"/>    	
	<xsl:apply-templates select="*" mode="ch-init"/>
    </ch-init>
</xsl:template>

<xsl:template match="varref">
    <varref>
        <xsl:apply-templates select="*|@line|@column"/>
    </varref>
</xsl:template>

<xsl:template match="array_index">
    <array-index>
        <xsl:apply-templates select="*|@line|@column"/>
    </array-index>
</xsl:template>

<xsl:template match="fifo_send">
    <send type="fifo">
        <xsl:apply-templates select="*|@line|@column"/>
    </send>
</xsl:template>

<xsl:template match="sorted_send">
    <send type="sorted">
        <xsl:apply-templates select="*|@line|@column"/>
    </send>
</xsl:template>

<xsl:template match="fifo_receive">
    <receive type="fifo">
        <xsl:apply-templates select="*|@line|@column"/>
    </receive>
</xsl:template>

<xsl:template match="selective_receive">
    <receive type="selective">
        <xsl:apply-templates select="*|@line|@column"/>
    </receive>
</xsl:template>

<xsl:template match="pollfifo_receive">
    <receive type="pollfifo">
        <xsl:apply-templates select="*|@line|@column"/>
    </receive>
</xsl:template>

<xsl:template match="pollselective_receive">
    <receive type="pollselective">
        <xsl:apply-templates select="*|@line|@column"/>
    </receive>
</xsl:template>

<xsl:template match="fifo_recv_poll">
    <recv-poll type="fifo">
        <xsl:apply-templates select="*|@line|@column"/>
    </recv-poll>
</xsl:template>

<xsl:template match="selective_recv_poll">
    <recv-poll type="selective">
        <xsl:apply-templates select="*|@line|@column"/>
    </recv-poll>
</xsl:template>

<xsl:template match="underscore_recv_arg">
    <underscore>
        <xsl:apply-templates select="*|@line|@column"/>
    </underscore>
</xsl:template>

<xsl:template match="eval_recv_arg">
    <eval>
        <xsl:apply-templates select="*|@line|@column"/>
    </eval>
</xsl:template>

<xsl:template match="const_recv_arg">
    <receive-const value="{t_const/@text}">
        <xsl:attribute name="neg">no</xsl:attribute>
        <xsl:apply-templates select="@line|@column"/>
  	<xsl:if test="t_minus">
            <xsl:attribute name="neg">yes</xsl:attribute>
        </xsl:if>
    </receive-const>
</xsl:template>

<xsl:template match="expr_assign">
    <assign>
        <xsl:apply-templates select="*|@line|@column"/>
    </assign>
</xsl:template>

<xsl:template match="underscore_assign">
    <underscore-assign>
        <xsl:apply-templates select="*|@line|@column"/>
    </underscore-assign>
</xsl:template>

<xsl:template match="inc_assign">
    <increment>
        <xsl:apply-templates select="*|@line|@column"/>
    </increment>
</xsl:template>

<xsl:template match="dec_assign">
    <decrement>
        <xsl:apply-templates select="*|@line|@column"/>
    </decrement>
</xsl:template>

<xsl:template match="if_stmnt">
    <if>
      <xsl:apply-templates select="*|@line|@column"/>
    </if>
</xsl:template>

<xsl:template match="do_stmnt">
    <do>
      <xsl:apply-templates select="*|@line|@column"/>
    </do>
</xsl:template>

<xsl:template match="atomic_stmnt">
    <atomic>
      <xsl:apply-templates select="*|@line|@column"/>
    </atomic>
</xsl:template>

<xsl:template match="step_stmnt">
    <d-step>
      <xsl:apply-templates select="*|@line|@column"/>
    </d-step>
</xsl:template>
 
<xsl:template match="sequence_stmnt">
    <block>
      <xsl:apply-templates select="*|@line|@column"/>
    </block>
</xsl:template>

<xsl:template match="else_stmnt">
    <else>
        <xsl:apply-templates select="@line|@column"/>
    </else>
</xsl:template>

<xsl:template match="break_stmnt">
    <break>
        <xsl:apply-templates select="@line|@column"/>
    </break>
</xsl:template>

<xsl:template match="goto_stmnt">
    <goto>
        <xsl:apply-templates select="*|@line|@column"/>
    </goto>
</xsl:template>

<xsl:template match="name_stmnt">
    <label>
        <xsl:apply-templates select="*|@line|@column"/>
    </label>
</xsl:template>
    
<xsl:template match="printf_stmnt">
    <printf>
        <xsl:attribute name="format">
            <xsl:variable name="format">
                <xsl:value-of select="t_string[1]/@text"/>
            </xsl:variable>
            <xsl:value-of select="substring( $format, 2, string-length( $format ) - 2 )"/>
        </xsl:attribute>
        <xsl:apply-templates select="*|@line|@column"/>
    </printf>
</xsl:template>

<xsl:template match="printm_stmnt">
    <printf format="%e">
        <xsl:apply-templates select="*|@line|@column"/>
    </printf>
</xsl:template>

<xsl:template match="assert_stmnt">
    <assert>
        <xsl:apply-templates select="*|@line|@column"/>
    </assert>
</xsl:template>

<xsl:template match="c_code_stmnt">
    <c-code>
        <xsl:apply-templates select="*|@line|@column"/>
    </c-code>
</xsl:template>

<xsl:template match="c_expr_stmnt">
    <c-expr>
        <xsl:apply-templates select="*|@line|@column"/>
    </c-expr>
</xsl:template>

<xsl:template match="c_assert">
    <c-assert>
        <xsl:apply-templates select="*|@line|@column"/>
    </c-assert>
</xsl:template>


<xsl:template match="options">
  <xsl:apply-templates mode="options"/>
</xsl:template>

<xsl:template match="sequence" mode="options">
  <option>
    <xsl:apply-templates select="*|@line|@column"/>
  </option>
</xsl:template>

<xsl:template match="binary_expr_binary_or">
    <or><xsl:apply-templates select="*|@line|@column"/></or>
</xsl:template>

<xsl:template match="binary_expr_binary_and">
    <and><xsl:apply-templates select="*|@line|@column"/></and>
</xsl:template>

<xsl:template match="binary_expr_binary_bitor">
    <bit-or><xsl:apply-templates select="*|@line|@column"/></bit-or>
</xsl:template>

<xsl:template match="binary_expr_binary_bitxor">
    <bit-xor><xsl:apply-templates select="*|@line|@column"/></bit-xor>
</xsl:template>

<xsl:template match="binary_expr_binary_bitand">
    <bit-and><xsl:apply-templates select="*|@line|@column"/></bit-and>
</xsl:template>

<xsl:template match="eq_expr_binary_eq">
    <eq><xsl:apply-templates select="*|@line|@column"/></eq>
</xsl:template>

<xsl:template match="ne_expr_binary_eq">
    <neq><xsl:apply-templates select="*|@line|@column"/></neq>
</xsl:template>

<xsl:template match="lt_expr_binary_ineq">
    <lt><xsl:apply-templates select="*|@line|@column"/></lt>
</xsl:template>

<xsl:template match="gt_expr_binary_ineq">
    <gt><xsl:apply-templates select="*|@line|@column"/></gt>
</xsl:template>

<xsl:template match="le_expr_binary_ineq">
    <lte><xsl:apply-templates select="*|@line|@column"/></lte>
</xsl:template>

<xsl:template match="ge_expr_binary_ineq">
    <gte><xsl:apply-templates select="*|@line|@column"/></gte>
</xsl:template>

<xsl:template match="left_expr_binary_shift">
    <ltlt><xsl:apply-templates select="*|@line|@column"/></ltlt>
</xsl:template>

<xsl:template match="right_expr_binary_shift">
    <gtgt><xsl:apply-templates select="*|@line|@column"/></gtgt>
</xsl:template>

<xsl:template match="plus_expr_binary_add">
    <add><xsl:apply-templates select="*|@line|@column"/></add>
</xsl:template>

<xsl:template match="minus_expr_binary_add">
    <sub><xsl:apply-templates select="*|@line|@column"/></sub>
</xsl:template>

<xsl:template match="star_expr_binary_mult">
    <mul><xsl:apply-templates select="*|@line|@column"/></mul>
</xsl:template>

<xsl:template match="div_expr_binary_mult">
    <div><xsl:apply-templates select="*|@line|@column"/></div>
</xsl:template>

<xsl:template match="mod_expr_binary_mult">
    <mod><xsl:apply-templates select="*|@line|@column"/></mod>
</xsl:template>

<xsl:template match="tilde_expr_unary">
   <tilde><xsl:apply-templates select="*|@line|@column"/></tilde>
</xsl:template>

<xsl:template match="exclam_expr_unary">
   <excl><xsl:apply-templates select="*|@line|@column"/></excl>
</xsl:template>

<xsl:template match="minus_expr_unary">
   <neg><xsl:apply-templates select="*|@line|@column"/></neg>
</xsl:template>

<xsl:template match="arrow_expr_simple">
   <arrow-colon><xsl:apply-templates select="*|@line|@column"/></arrow-colon>
</xsl:template>

<xsl:template match="len_expr_simple">
   <len><xsl:apply-templates select="*|@line|@column"/></len>
</xsl:template>

<xsl:template match="const_expr_simple">
   <const-expr value="{t_const/@text}" line="{@line}" column="{@column}">
      <xsl:apply-templates select="@line|@column"/>
   </const-expr>
</xsl:template>

<xsl:template match="timeout_expr_simple">
   <timeout><xsl:apply-templates select="@line|@column"/></timeout>
</xsl:template>

<xsl:template match="u_pid_expr_simple">
   <pid><xsl:apply-templates select="@line|@column"/></pid>
</xsl:template>

<xsl:template match="u_nr_pr_expr_simple">
   <nr_pr><xsl:apply-templates select="@line|@column"/></nr_pr>
</xsl:template>

<xsl:template match="u_last_expr_simple">
   <last><xsl:apply-templates select="@line|@column"/></last>
</xsl:template>

<xsl:template match="np_expr_simple">
   <np><xsl:apply-templates select="@line|@column"/></np>
</xsl:template>

<xsl:template match="enabled_expr_simple">
   <enabled><xsl:apply-templates select="*|@line|@column"/></enabled>
</xsl:template>

<xsl:template match="pc_value_expr_simple">
   <pc-value><xsl:apply-templates select="*|@line|@column"/></pc-value>
</xsl:template>

<xsl:template match="r_ref_label_expr_simple">
   <r-ref-label proc="{t_name[1]/@text}" name="{t_name[2]/@text}">
      <xsl:apply-templates select="@line|@column|*[name(.) != 't_name']"/>
   </r-ref-label>
</xsl:template>

<xsl:template match="r_ref_var_expr_simple">
   <r-ref-var proc="{t_name[1]/@text}" name="{t_name[2]/@text}">
      <xsl:apply-templates select="@line|@column|*[name(.) != 't_name']"/>
   </r-ref-var>
</xsl:template>

<xsl:template match="run_expr_simple">
   <run><xsl:apply-templates select="*|@line|@column"/></run>
</xsl:template>

<xsl:template match="chanop_expr_simple">
   <chanop><xsl:apply-templates select="*|@line|@column"/></chanop>
</xsl:template>

<xsl:template match="full_chanop">
   <xsl:attribute name="type">full</xsl:attribute>
</xsl:template>

<xsl:template match="empty_chanop">
   <xsl:attribute name="type">empty</xsl:attribute>
</xsl:template>

<xsl:template match="nfull_chanop">
   <xsl:attribute name="type">nfull</xsl:attribute>
</xsl:template>

<xsl:template match="nempty_chanop">
   <xsl:attribute name="type">nempty</xsl:attribute>
</xsl:template>

<xsl:template match="t_name">
    <xsl:attribute name="name">
        <xsl:value-of select="@text"/>
    </xsl:attribute>
</xsl:template>

<xsl:template match="inline_decl">
    <inline-decl>
        <xsl:apply-templates select="*|@line|@column"/>
    </inline-decl>
</xsl:template>

<xsl:template match="inline_prms">
    <inline-prms>
        <xsl:apply-templates select="*|@line|@column"/>
    </inline-prms>
</xsl:template>

<xsl:template match="inline_prm">
    <inline-prm>
        <xsl:apply-templates select="*|@line|@column"/>
    </inline-prm>
</xsl:template>

<xsl:template match="inline_body">
    <block>
        <xsl:apply-templates select="*|@line|@column"/>
    </block>
</xsl:template>

<xsl:template match="inline_stmnt">
    <inline-stmnt>
        <xsl:apply-templates select="*|@line|@column"/>
    </inline-stmnt>
</xsl:template>

<xsl:template match="t_source_code_brace">
    <embedded-c>
	<xsl:apply-templates select="@line|@column"/>
	<xsl:value-of select="@text"/>
    </embedded-c>
</xsl:template>

<xsl:template match="t_source_code_bracket">
    <embedded-c>
        <xsl:apply-templates select="@line|@column"/>
        <xsl:value-of select="@text"/>
    </embedded-c>
</xsl:template>


</xsl:stylesheet>

